-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.keccak_pkg.all;

-- asynchronous memory for Keccak constants
-- possible combinations for generic unrolled round factor ur = {1, 2, 3, 4, 6, 8, 12, 24}
entity keccak_cons is
generic (ur : integer:=1);
port(
	addr	 		: in  std_logic_vector(7 downto 0);
	rc				: out std_logic_vector(7 downto 0));
end keccak_cons;
    
--architecture keccak_cons of keccak_cons is
--
----constant memory: matrix :=(
----x"0000000000000001", x"0000000000008082", x"800000000000808A", x"8000000080008000",
----x"000000000000808B", x"0000000080000001", x"8000000080008081", x"8000000000008009",
----x"000000000000008A", x"0000000000000088", x"0000000080008009", x"000000008000000A",
----x"000000008000808B", x"800000000000008B", x"8000000000008089", x"8000000000008003",
----x"8000000000008002", x"8000000000000080", x"000000000000800A", x"800000008000000A",
----x"8000000080008081", x"8000000000008080", x"0000000080000001", x"8000000080008008",
----x"0000000000000000", x"0000000000000000", x"0000000000000000", x"0000000000000000",
----x"0000000000000000", x"0000000000000000", x"0000000000000000", x"0000000000000000");	  
--type matrix is array (0 to 31) of std_logic_vector(7 downto 0);
--constant my_rom0 : matrix :=(
--x"01", x"82", x"8A", x"00", 
--x"8B", x"01", x"81", x"09", 
--x"8A", x"88", x"09", x"0A", 
--x"8B", x"8B", x"89", x"03", 
--x"02", x"80", x"0A", x"0A", 
--x"81", x"80", x"01", x"08", 
--x"00", x"00", x"00", x"00", 
--x"00", x"00", x"00", x"00" );	
--
--constant my_rom1 : matrix := ( 
--x"00",x"80",x"80",x"80",
--x"80",x"00",x"80",x"80",
--x"00",x"00",x"80",x"00",
--x"80",x"00",x"80",x"80",
--x"80",x"00",x"80",x"00",
--x"80",x"80",x"00",x"80",
--x"00",x"00",x"00",x"00",
--x"00",x"00",x"00",x"00" );
--					 
--constant my_rom3 : matrix := ( 
--x"00",x"00",x"00",x"80",
--x"00",x"80",x"80",x"00",
--x"00",x"00",x"80",x"80",
--x"80",x"00",x"00",x"00",
--x"00",x"00",x"00",x"80",
--x"80",x"00",x"80",x"80",
--x"00",x"00",x"00",x"00",
--x"00",x"00",x"00",x"00" );
--
--constant my_rom7 : matrix := ( 
--x"00",x"00",x"80",x"80",
--x"00",x"00",x"80",x"80",
--x"00",x"00",x"00",x"00",
--x"00",x"80",x"80",x"80",
--x"80",x"80",x"00",x"80",
--x"80",x"80",x"00",x"80",
--x"00",x"00",x"00",x"00",
--x"00",x"00",x"00",x"00" );
--
--begin
--	
--	with addr(2 downto 0) select
--	rc <= my_rom0(conv_integer(unsigned(addr(7 downto 3)))) when "000",
--		  my_rom1(conv_integer(unsigned(addr(7 downto 3)))) when "001",
--		  my_rom3(conv_integer(unsigned(addr(7 downto 3)))) when "011",
--		  my_rom7(conv_integer(unsigned(addr(7 downto 3)))) when "111",
--		  											  x"00" when others;
--
--end keccak_cons;

                  


architecture keccak_cons of keccak_cons is

type byteArray is array (0 to 31) of std_logic_vector(7 downto 0);
constant my_rom0 : byteArray :=(
x"01", x"82", x"8A", x"00", 
x"8B", x"01", x"81", x"09", 
x"8A", x"88", x"09", x"0A", 
x"8B", x"8B", x"89", x"03", 
x"02", x"80", x"0A", x"0A", 
x"81", x"80", x"01", x"08", 
x"00", x"00", x"00", x"00", 
x"00", x"00", x"00", x"00" );	

type bitArray is array (0 to 31) of std_logic;

constant my_rom1 : bitArray := ( 
'0','1','1','1',
'1','0','1','1',
'0','0','1','0',
'1','0','1','1',
'1','0','1','0',
'1','1','0','1',
'0','0','0','0',
'0','0','0','0' );
					 
constant my_rom3 : bitArray := ( 
'0','0','0','1',
'0','1','1','0',
'0','0','1','1',
'1','0','0','0',
'0','0','0','1',
'1','0','1','1',
'0','0','0','0',
'0','0','0','0' );

constant my_rom7 : bitArray := ( 
'0','0','1','1',
'0','0','1','1',
'0','0','0','0',
'0','1','1','1',
'1','1','0','1',
'1','1','0','1',
'0','0','0','0',
'0','0','0','0' );

begin
	
	with addr(2 downto 0) select
	rc <= my_rom0(conv_integer(unsigned(addr(7 downto 3)))) when "000",
		  my_rom1(conv_integer(unsigned(addr(7 downto 3)))) & "0000000" when "001",
		  my_rom3(conv_integer(unsigned(addr(7 downto 3)))) & "0000000" when "011",
		  my_rom7(conv_integer(unsigned(addr(7 downto 3)))) & "0000000" when "111",
		  											  x"00" when others;

end keccak_cons;