-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;


package keccak_pkg is
	 -- Keccak parameters 
	constant KECCAK_STATE 			: integer := 1600;	
	constant KECCAK256_CAPACITY 	: integer := 1088;
	constant KECCAK512_CAPACITY 	: integer := 576;
	
	-- width of the interface ports					 
	constant iw						: integer := 40;   -- input word
	constant ow						: integer := 8;   -- output word

			 
	-- number of rounds of Keccak 		 
	constant roundnr256	 			: integer := 24;
	constant roundnr_final 			: integer := 1;	 
	
	type sub_field_type is array (0 to 4) of std_logic_vector(7 downto 0);										 
	type field_type is array (0 to 4) of sub_field_type;	   
	
	function field2str ( field : field_type ) return std_logic_vector;
	function str2field ( field : std_logic_vector) return field_type;	
end keccak_pkg;

package body keccak_pkg is	
	function field2str ( field : field_type ) return std_logic_vector is
		variable ret : std_logic_vector(199 downto 0);
	begin			 
		for i in 0 to 4 loop		  
			for j in 0 to 4 loop
				ret(200-i*40-j*8-1 downto 200-i*40-j*8-8) := field(i)(j);
			end loop;
		end loop;		
		return ret;
	end function field2str;	
	
	function str2field ( field : std_logic_vector ) return field_type is
		variable ret : field_type;
	begin			 
		for i in 0 to 4 loop		  
			for j in 0 to 4 loop
				ret(i)(j) := field(200-i*40-j*8-1 downto 200-i*40-j*8-8);
			end loop;
		end loop;		
		return ret;
	end function str2field;	
	
end package body keccak_pkg;