-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

-- Possible generics values: 
-- HS = {HASH_SIZE_256, HASH_SIZE_512} 

library ieee;
use ieee.std_logic_1164.all;	
use work.sha3_pkg.all;
use work.keccak_pkg.all;


entity keccak_top is
generic ( HS : integer := HASH_SIZE_256 );
port (		
		rst 			: in std_logic;
		clk 			: in std_logic;
		src_ready 		: in std_logic;
		src_read  		: out std_logic;
		dst_ready 		: in std_logic;
		dst_write 		: out std_logic;		
		din				: in std_logic_vector(39 downto 0);
		dout			: out std_logic_vector(7 downto 0));	   
end keccak_top;


architecture structure of keccak_top is 	 
    
	signal c:  std_logic_vector(31 downto 0);
	signal loadLen, decLen : std_logic;
    signal selCI, selTRP, selLastByte, enSaveRotatedRho : std_logic;
    signal selAbsorb, rstMem, selWriteMem, selReadMem, selWriteShift : std_logic;	
    signal writeLane : std_logic_vector(4 downto 0);
    signal rc_addr : std_logic_vector(7 downto 0);
    
begin
	
	H256Gen: if HS = HASH_SIZE_256 generate
		control_gen : entity work.keccak_control(h256)
			port map (clk =>clk, rst=>rst, 
			c => c, loadLen => loadLen, decLen => decLen,
			final_segment => din(39), src_ready => src_ready, dst_ready => dst_ready, src_read => src_read, dst_write => dst_write, 
	        selCI => selCI, selTRP => selTRP, selLastByte => selLastByte, enSaveRotatedRho => enSaveRotatedRho,
	        selAbsorb => selAbsorb, selWriteShift => selWriteShift, rstMem => rstMem, selWriteMem => selWriteMem, selReadMem => selReadMem,
	        writeLane => writeLane, rc_addr => rc_addr ); 							
	end generate;	 
	H512Gen: if HS = HASH_SIZE_512 generate
		control_gen : entity work.keccak_control(h512)
			port map (clk =>clk, rst=>rst, 
			c => c, loadLen => loadLen, decLen => decLen,
			final_segment => din(39), src_ready => src_ready, dst_ready => dst_ready, src_read => src_read, dst_write => dst_write, 
	        selCI => selCI, selTRP => selTRP, selLastByte => selLastByte, enSaveRotatedRho => enSaveRotatedRho,
	        selAbsorb => selAbsorb, selWriteShift => selWriteShift, rstMem => rstMem, selWriteMem => selWriteMem, selReadMem => selReadMem,
	        writeLane => writeLane, rc_addr => rc_addr ); 							
	end generate;


	datapath_gen : entity work.keccak_datapath(struct)	
		generic map ( HS => HS )
		port map (clk => clk, din => din, dout => dout, 
        c => c, loadLen => loadLen, decLen => decLen, 
        selCI => selCI, selTRP => selTRP, selLastByte => selLastByte, enSaveRotatedRho => enSaveRotatedRho,
        selAbsorb => selAbsorb, selWriteShift => selWriteShift, rstMem => rstMem, selWriteMem => selWriteMem, selReadMem => selReadMem,
        writeLane => writeLane, rc_addr => rc_addr );

	
end structure;		

